==> mongodb-5.0.32.gh.tar.gz: mongo-r5.0.32/src/mongo/db/storage/temporary_record_store.h <==
/**
 *    Copyright (C) 2018-present MongoDB, Inc.
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the Server Side Public License, version 1,
 *    as published by MongoDB, Inc.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    Server Side Public License for more details.
 *
 *    You should have received a copy of the Server Side Public License
 *    along with this program. If not, see
 *    <http://www.mongodb.com/licensing/server-side-public-license>.
 *
 *    As a special exception, the copyright holders give permission to link the
 *    code of portions of this program with the OpenSSL library under certain
 *    conditions as described in each individual source file and distribute
 *    linked combinations including the program with the OpenSSL library. You
 *    must comply with the Server Side Public License in all respects for
 *    all of the code used other than as permitted herein. If you modify file(s)
 *    with this exception, you may extend this exception to your version of the
 *    file(s), but you are not obligated to do so. If you do not wish to do so,
 *    delete this exception statement from your version. If you delete this
 *    exception statement from all source files in the program, then also delete
 *    it in the license file.
 */

#pragma once

#include "mongo/db/storage/record_store.h"

namespace mongo {

/**
 * Manages the lifetime of a temporary RecordStore.
 *
 * Derived classes must implement and call finalizeTemporaryTable() before destruction to delete
 * the underlying RecordStore from the storage engine or to keep the underlying RecordStore.
 */
class TemporaryRecordStore {
public:
    enum class FinalizationAction { kDelete, kKeep };

    TemporaryRecordStore(std::unique_ptr<RecordStore> rs) : _rs(std::move(rs)) {}

    // Not copyable.
    TemporaryRecordStore(const TemporaryRecordStore&) = delete;
    TemporaryRecordStore& operator=(const TemporaryRecordStore&) = delete;

    // Move constructor.
    TemporaryRecordStore(TemporaryRecordStore&& other) noexcept : _rs(std::move(other._rs)) {}

    virtual ~TemporaryRecordStore() {}

    virtual void finalizeTemporaryTable(OperationContext* opCtx, FinalizationAction action) {}

    RecordStore* rs() {
        return _rs.get();
    }

    const RecordStore* rs() const {
        return _rs.get();
    }

protected:
    std::unique_ptr<RecordStore> _rs;
};
}  // namespace mongo

==> mongodb-6.0.27.gh.tar.gz: mongo-r6.0.27/src/mongo/db/storage/temporary_record_store.h <==
/**
 *    Copyright (C) 2018-present MongoDB, Inc.
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the Server Side Public License, version 1,
 *    as published by MongoDB, Inc.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    Server Side Public License for more details.
 *
 *    You should have received a copy of the Server Side Public License
 *    along with this program. If not, see
 *    <http://www.mongodb.com/licensing/server-side-public-license>.
 *
 *    As a special exception, the copyright holders give permission to link the
 *    code of portions of this program with the OpenSSL library under certain
 *    conditions as described in each individual source file and distribute
 *    linked combinations including the program with the OpenSSL library. You
 *    must comply with the Server Side Public License in all respects for
 *    all of the code used other than as permitted herein. If you modify file(s)
 *    with this exception, you may extend this exception to your version of the
 *    file(s), but you are not obligated to do so. If you do not wish to do so,
 *    delete this exception statement from your version. If you delete this
 *    exception statement from all source files in the program, then also delete
 *    it in the license file.
 */

#pragma once

#include "mongo/db/storage/record_store.h"

namespace mongo {

/**
 * Manages the lifetime of a temporary RecordStore. Unless keep() is called, the managed RecordStore
 * will be dropped after destruction.
 */
class TemporaryRecordStore {
public:
    TemporaryRecordStore(std::unique_ptr<RecordStore> rs) : _rs(std::move(rs)){};
    virtual ~TemporaryRecordStore() {}

    // Not copyable.
    TemporaryRecordStore(const TemporaryRecordStore&) = delete;
    TemporaryRecordStore& operator=(const TemporaryRecordStore&) = delete;

    // Move constructor.
    TemporaryRecordStore(TemporaryRecordStore&& other) = default;

    // If called, the RecordStore will not be dropped when the TemporaryRecordStore is destructed.
    void keep() {
        _keep = true;
    }

    RecordStore* rs() {
        return _rs.get();
    }

    const RecordStore* rs() const {
        return _rs.get();
    }

protected:
    std::unique_ptr<RecordStore> _rs;
    bool _keep = false;
};
}  // namespace mongo
